﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace GameLoop_CompositionTarget
{
	/// <summary>
	/// MainWindow.xaml에 대한 상호 작용 논리
	/// </summary>
	public partial class MainWindow : Window
	{
		/// <summary>
		/// 게임 루프용 클래스
		/// </summary>
		private GameLoop m_insGameLoop;
		/// <summary>
		/// 키보드 핸들러
		/// </summary>
		private KeyHandler keyHandler;
		/// <summary>
		/// 회전속도
		/// </summary>
		private double rotationSpeed = 150;

		/// <summary>
		/// 루프회전수 체크
		/// </summary>
		private long m_long = 0;


		//☆☆☆☆☆ FPS용 ☆☆☆☆☆
		/// <summary>
		/// 경과시간
		/// </summary>
		private double m_dElapsed;
		/// <summary>
		/// 총 경과 시간
		/// </summary>
		private double m_dTotalElapsed;
		/// <summary>
		/// 마지막 틱
		/// </summary>
		private int m_nLastTick;
		/// <summary>
		/// 현재 틱
		/// </summary>
		private int m_nCurrentTick;
		/// <summary>
		/// 프레임 카운트(이 녀석이 최종 FPS가 됨)
		/// </summary>
		private int m_nFrameCount;
		/// <summary>
		/// 프레임 카운트 시간(1초동안 카운트가 되었는지 확인)
		/// </summary>
		private double m_dFrameCountTime;
		/// <summary>
		/// 프래임 결과를 임시 보관하는 변수
		/// </summary>
		private int m_nFrameRate;
		//☆☆☆☆☆ 여기까지 - FPS용 ☆☆☆☆☆

		public MainWindow()
		{
			InitializeComponent();


			this.GotFocus += new RoutedEventHandler(Page_GotFocus);
			this.LostFocus += new RoutedEventHandler(Page_LostFocus);
			this.MouseLeftButtonDown += new MouseButtonEventHandler(Page_MouseLeftButtonDown);

			m_insGameLoop = new GameLoop();
			m_insGameLoop.OnUpdate += new GameLoop.UpdateHandler(gameLoop_Update);

			keyHandler = new KeyHandler(this);

			//마지막 틱을 설정해준다.
			this.m_nLastTick = Environment.TickCount;

		}

		void Page_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
		{
			clickToStart.Visibility = Visibility.Collapsed;
			m_insGameLoop.Start();
		}

		void Page_LostFocus(object sender, RoutedEventArgs e)
		{
			clickToStart.Visibility = Visibility.Visible;
			m_insGameLoop.Stop();
		}

		void Page_GotFocus(object sender, RoutedEventArgs e)
		{
			clickToStart.Visibility = Visibility.Collapsed;
			m_insGameLoop.Start();
		}

		void gameLoop_Update(object sender, TimeSpan elapsed)
		{

			this.m_nCurrentTick = Environment.TickCount;
			this.m_dElapsed = (double)(this.m_nCurrentTick - this.m_nLastTick) / 1000.0;
			this.m_dTotalElapsed += this.m_dElapsed;
			this.m_nLastTick = this.m_nCurrentTick;

			m_nFrameCount++;
			m_dFrameCountTime += this.m_dElapsed;
			if (m_dFrameCountTime >= 1.0)
			{
				m_dFrameCountTime -= 1.0;
				m_nFrameRate = m_nFrameCount;
				m_nFrameCount = 0;
				test1.Text = "FPS: " + m_nFrameRate.ToString();
			}

			//
			if (keyHandler.IsKeyPressed(Key.A) || keyHandler.IsKeyPressed(Key.Left))
			{
				ship.RotationAngle -= rotationSpeed * elapsed.TotalSeconds;
			}
			else if (keyHandler.IsKeyPressed(Key.D) || keyHandler.IsKeyPressed(Key.Right))
			{
				ship.RotationAngle += rotationSpeed * elapsed.TotalSeconds;
			}

			test2.Text = (++m_long / 60).ToString();

		}

	}
}
